#include "inkey.ch"

#define K_DBF              1
#define K_NTX              2
#define K_REP              3
#define K_SELECTED         4
#define K_COLOR            5
#define K_WORK_AREA        6

#define K_THE_ARRAY        1                     && reference to the array to browse
#define K_CURRENT_ELEMENT  2                     && NUMBER of the current array element

#define K_LIST_O_FILES     1
#define K_CURRENT_SELECT   2
#define K_CURRENT_RECNO    3

#define K_CARGO_ELEMENTS   3

STATIC oBrArray
STATIC oBr

FUNCTION main()
  LOCAL nRow    := ROW()
  LOCAL nCol    := COL()
  LOCAL cScreen := SAVESCREEN(0, 0, MAXROW(), MAXCOL() )

  SETCOLOR("+BG/B")
  CLEAR

  SET CURSOR OFF
  LoadArrayBrowse()
  ArrayProcess()
  SET CURSOR ON

  CLOSE DATABASES
  RESTSCREEN(0, 0, MAXROW(), MAXCOL(), cScreen)
  SET COLOR TO
  DEVPOS(nRow, nCol)
RETURN(NIL)

***********
* Function: LoadArrayBrowse()
* Author..: Troy Henikoff
* Date....: 10/18/92
* Notes...: Loads up the TBrowse of an array of "files"
* Returns.: NIL
***********

FUNCTION LoadArrayBrowse()
  LOCAL oColumn

  oBrArray       := TBrowseNew(4, 15, 19, 65 )
  oBrArray:Cargo := ARRAY(K_CARGO_ELEMENTS)
  oBrArray:Cargo[K_THE_ARRAY]       := {}
  oBrArray:Cargo[K_CURRENT_ELEMENT] := 1
  oBrArray:ColorSpec  := "BG/N, N/W"
  oBrArray:SkipBlock  := {|x| ArraySkipper(x) }  && assign it a SkipBlock


  USE Files NEW SHARED
  DO WHILE .NOT. EOF()
    AADD(oBrArray:Cargo[K_THE_ARRAY], {Files->Dbf_file, Files->Ntx_file, Files->Rep, .F., Files->Color, 0} )
    oBrArray:ColorSpec  := oBrArray:ColorSpec + ","+Files->Color
    SKIP
  ENDDO
  CLOSE DATABASES

  oColumn := TBColumnNew("  Rep----------------  File--------------------------",;
  {|| IF(oBrArray:Cargo[K_THE_ARRAY][ oBrArray:Cargo[K_CURRENT_ELEMENT] ][K_SELECTED], "", " ") + " " +;
  oBrArray:Cargo[K_THE_ARRAY][ oBrArray:Cargo[K_CURRENT_ELEMENT] ][K_REP]+" "+;
  LEFT(oBrArray:Cargo[K_THE_ARRAY][ oBrArray:Cargo[K_CURRENT_ELEMENT] ][K_DBF],30) } )

  *  oColumn:ColorBlock := {|| IF(oBrArray:Cargo[K_THE_ARRAY][ oBrArray:Cargo[K_CURRENT_ELEMENT] ][K_SELECTED], {2,3}, {1,3}) }

  oColumn:ColorBlock := {|| { oBrArray:Cargo[K_CURRENT_ELEMENT]+2, 2 }  }

  oBrArray:AddColumn(oColumn)

  @ 1, 20 SAY PADC("BROWSE OF FILES",                  40 ) COLOR "+GR/B"
  @22, 20 SAY PADC(":Select    F5:Browse Files", 40 ) COLOR "+W/B"

RETURN(NIL)

***********
* Function: ArraySkipper()
* Author..: Troy Henikoff
* Date....: 10/18/92
* Notes...: Skipper function for TBrowsing arrays
*
* Params..: nToSkip - NUMBER to skip
*
* Returns.: NUMBER skipped
***********

FUNCTION ArraySkipper(nToSkip)
  LOCAL nSkipped := 0
  LOCAL aArray := oBrArray:Cargo[K_THE_ARRAY]

  DO CASE
    CASE nToSkip == 0
      *** DO Nothing, Skip 0!!!! ***

    CASE nToSkip > 0
      DO WHILE ( nSkipped < nToSkip ) .AND. oBrArray:Cargo[K_CURRENT_ELEMENT] < LEN(aArray)
        nSkipped++
        oBrArray:Cargo[K_CURRENT_ELEMENT]++
      ENDDO

    OTHERWISE
      DO WHILE ( nSkipped > nToSkip ) .AND. oBrArray:Cargo[K_CURRENT_ELEMENT] > 1
        nSkipped--
        oBrArray:Cargo[K_CURRENT_ELEMENT]--
      ENDDO

  ENDCASE
RETURN(nSkipped)

***********
* Function: ArrayProcess
* Author..: Troy Henikoff
* Date....: 10/18/92
* Notes...: Processes keystrokes for a TBrowse of an array
* Returns.: NIL
***********

FUNCTION ArrayProcess()
  LOCAL nKey                                     && Key that the user presses
  LOCAL cScreen

  DO WHILE .T.
    nKey := 0                                    && Reset value of nKey

    *** stabilize the display (standard TBrowse stuff) ***
    DO WHILE .NOT. oBrArray:stabilize()          && DO while the display is not stabilized, incrementally stabilize it.
      nKey := INKEY()                            &&   check to see if the user has pressed a key
      IF nKey <> 0                               &&   IF a key was pressed
        EXIT                                     &&     EXIT out of this loop and process the user's request
      ENDIF                                      &&   ENDIF
    ENDDO                                        && ENDDO - go back and stabilize it a little more...

    IF oBrArray:stable .AND. nKey==0             && IF the display is stable
      nKey := INKEY(0)                           &&   Wait forever for the user to hit a key
    ENDIF                                        && ENDIF

    *** Finally, we get to process the key that was pressed ***
    DO CASE
      CASE nKey == K_DOWN
        oBrArray:down()

      CASE nKey == K_UP
        oBrArray:up()

      CASE nKey == K_PGDN
        oBrArray:pageDown()

      CASE nKey == K_PGUP
        oBrArray:pageUp()

      CASE nKey == K_ESC
        EXIT

      CASE nKey == K_ENTER .AND. oBrArray:Stable
        oBrArray:Cargo[K_THE_ARRAY][ oBrArray:Cargo[K_CURRENT_ELEMENT] ][K_SELECTED] := .NOT. oBrArray:Cargo[K_THE_ARRAY][ oBrArray:Cargo[K_CURRENT_ELEMENT] ][K_SELECTED]
        oBrArray:RefreshCurrent()

      CASE nKey == K_F5
        cScreen := SAVESCREEN(0, 0, MAXROW(), MAXCOL() )

        IF LoadBrowse()
          KeyProcess()
        ELSE
          TONE(300,10)
        ENDIF

        RESTSCREEN(0, 0, MAXROW(), MAXCOL(), cScreen)

    ENDCASE
  ENDDO

RETURN(NIL)


***********
* Function: LoadBrowse()
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Create & loads a TBrowse object
* Returns.: LOGICAL - .T. if at least one file got opened
***********

FUNCTION LoadBrowse()
  LOCAL nElement, oColumn
  LOCAL lSomeOpen := .F.                         && flag that some files were opened...

  SETCOLOR("+BG/B")
  CLEAR
  SETCOLOR("+BG/N")
  CLOSE DATABASES

  oBr                       := TBrowseNew(4, 15, 18, 75 )
  oBr:Cargo                 := ARRAY(K_CARGO_ELEMENTS)
  oBr:Cargo[K_LIST_O_FILES] := {}
  oBr:ColorSpec             := "BG/N, N/W"

  FOR nElement := 1 TO LEN(oBrArray:Cargo[K_THE_ARRAY])
    IF oBrArray:Cargo[K_THE_ARRAY][nElement][K_SELECTED]
      AADD(oBr:Cargo[K_LIST_O_FILES], oBrArray:Cargo[K_THE_ARRAY][nElement])
    ENDIF
  NEXT nElement

  FOR nElement := 1 TO LEN(oBr:Cargo[K_LIST_O_FILES])
    *** Open the file ***
    SELECT (nElement)
    USE ( oBr:Cargo[K_LIST_O_FILES][nElement][K_DBF] ) INDEX ( oBr:Cargo[K_LIST_O_FILES][nElement][K_NTX] ) SHARED
    lSomeOpen := .T.
    oBr:Cargo[K_LIST_O_FILES][nElement][K_WORK_AREA] := SELECT()  && stuff the work area in new, expanded element
    oBr:ColorSpec := oBr:ColorSpec +","+oBr:Cargo[K_LIST_O_FILES][nElement][K_COLOR]
  NEXT nElement

  oBr:SkipBlock             := {|x| MultiFileSkipper(x) }  && assign it a SkipBlock
  oColumn := TBColumnNew("Cust--   Name-----------------", {|| DispCustomer() } )
  oColumn:ColorBlock := {|| {SELECT()+2, 2} }
  oBr:AddColumn(oColumn)

  IF lSomeOpen
    @ 1, 20 SAY PADC("BROWSE OF MULTIPLE CONTACT FILES",    40 ) COLOR "+GR/B"
    GoMultiTop()                                 && first record of the series
  ENDIF

RETURN(lSomeOpen)


***********
* Function: MultiFileSkipper()
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: SkipBlock for Multi-line TBrowse
*
* Params..: nToSkip - NUMBER of records to skip
*
* Returns.: NUMBER of records skipped
***********

FUNCTION MultiFileSkipper(nToSkip)
  LOCAL nSkipped, nLastGoodRec

  nSkipped   := 0                                && The number actually skipped
  nLastGoodRec := RECNO()                        && Keep the RECNO() of the last "good" record that was found

  DO CASE
    CASE nToSkip == 0                            && SKIP 0 is easy!
      SKIP 0

    CASE nToSkip > 0                             && CASE skip foreward
      DO WHILE nSkipped < nToSkip                && Try to skip the requested number of times
        IF .NOT. NextMultiRecord()               && IF cannot move to next record
          EXIT                                   &&   EXIT out of the loop, no more to look at
        ENDIF                                    && ENDIF
        nSkipped++                               &&   increment the number of records successfully skipped
      ENDDO

    OTHERWISE                                    && OTHERWISE skip backwards
      DO WHILE nSkipped > nToSkip                && Try to skip the requested number of times
        IF .NOT. PrevMultiRecord()               && IF cannot move to next record
          EXIT                                   &&   EXIT out of the loop, no more to look at
        ENDIF                                    && ENDIF
        nSkipped--                               &&   decrement the number of records successfully skipped
      ENDDO
  ENDCASE

RETURN(nSkipped)


***********
* Function: KeyProcess
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Processes keystrokes for a TBrowse
*
* Params..: none
*
* Returns.: NIL
***********

FUNCTION KeyProcess()
  LOCAL nKey                                     && Key that the user presses

  DO WHILE .T.
    nKey := 0                                    && Reset value of nKey

    *** stabilize the display (standard TBrowse stuff) ***
    DO WHILE .NOT. oBr:stabilize()               && DO while the display is not stabilized, incrementally stabilize it.
      nKey := INKEY()                            &&   check to see if the user has pressed a key
      IF nKey <> 0                               &&   IF a key was pressed
        EXIT                                     &&     EXIT out of this loop and process the user's request
      ENDIF                                      &&   ENDIF
    ENDDO                                        && ENDDO - go back and stabilize it a little more...

    *** This part is needed for multiuser applications to ensure that we are displaying the most current version of the data ***
    IF nKey == 0                                 && IF we disable caching AND the browse was completely re-drawn without a user request
      oBr:RefreshCurrent()                       &&   mark that the "Current" line needs to be refreshed
      DO WHILE .NOT. oBr:stabilize()             &&   re-draw (hopefully just 1 line)
      ENDDO                                      &&   no need to check keyboard etc.
    ENDIF                                        && ENDIF

    IF oBr:stable .AND. nKey==0                  && IF the display is stable
      nKey := INKEY(0)                           &&   Wait forever for the user to hit a key
    ENDIF                                        && ENDIF

    *** Finally, we get to process the key that was pressed ***
    DO CASE
      CASE nKey == K_DOWN
        oBr:down()

      CASE nKey == K_UP
        oBr:up()

      CASE nKey == K_PGDN
        oBr:pageDown()

      CASE nKey == K_PGUP
        oBr:pageUp()

      CASE nKey == K_ESC
        EXIT

      CASE nKey == K_ENTER
        EditCustomer() 
        AdjustRecnos()
        oBr:RefreshAll()                         && re-draw entire TBrowse after editing the record

    ENDCASE
  ENDDO
RETURN(NIL)

***********
* Function: EditCustomer()
* Author..: Troy Henkoff
* Date....: 10/17/92
* Notes...: edits on customer record
* Returns.: NIL
***********

FUNCTION EditCustomer()
  LOCAL cScreen := SAVESCREEN(0, 0, MAXROW(), MAXCOL() )
  LOCAL cColor  := SETCOLOR( "+W/N" )
  LOCAL getlist := {}
  LOCAL nRecno

  SET CURSOR ON
  @ 17, 1, 23, 49 BOX "Ŀ "          COLOR " BG/N"  && Draw pretty box for user
  @ 17, 4 SAY " Editing Customer Record "  COLOR "+GR/N"  && put a title on it

  SELECT (oBr:Cargo[K_CURRENT_SELECT])
  nRecno := RECNO()
  GOTO oBr:Cargo[K_CURRENT_RECNO]
  IF rlock()                                     && IF we can lock the record
    @ 18,  2 SAY "Customer......:" GET Customer PICTURE "999999"  &&   Let user edit data...
    @ 19,  2 SAY "Name..........:" GET Name     PICTURE "@!"
    @ 20,  2 SAY "Address.......:" GET Addr_1
    @ 21,  2 SAY "              :" GET Addr_2
    @ 22,  2 SAY "City, St, Zip.:" GET City
    @ 22, 34                       GET State    PICTURE "!!"
    @ 22, 38                       GET Zip      PICTURE "99999-9999"
    READ
    COMMIT                                       &&   NOTE: Flush Clipper's buffers BEFORE UNLOCK
    UNLOCK                                       &&   UNLOCK the record
  ELSE                                           && ELSE
    @20,10 SAY "Record not available..." COLOR "+R/N"  &&   Someone else has the record...
    ?? CHR(7)
    INKEY(3)
  ENDIF                                          && ENDIF
  GOTO nRecno
  RESTSCREEN(0, 0, MAXROW(), MAXCOL(), cScreen)  && restore the screen, etc...
  SETCOLOR(cColor)
  SET CURSOR OFF

RETURN(NIL)


***********
* Function: GoMultiTop()
* Author..: Troy Henikoff
* Date....: 10/18/92
* Notes...: Multi-file version of GOTO TOP
* Returns.: NIL
***********

FUNCTION GoMultiTop()
  LOCAL nElement
  LOCAL cMinValue:=CHR(255)                      && smallest value so far
  LOCAL nTopElement                              && array element with smallest value

  FOR nElement := 1 TO LEN(oBr:Cargo[K_LIST_O_FILES])  && for each file
    SELECT (oBr:Cargo[K_LIST_O_FILES][nElement][K_WORK_AREA])  && select work area
    GOTO TOP                                     && goto top of file
    IF UPPER(Name) < cMinValue                   && if it is the best answer so far
      cMinValue := UPPER(Name)                   &&   Store the info about it
      nTopElement := nElement
    ENDIF
  NEXT nElement

  *** set up record pointers so that we are pointing to the smallest record ***
  oBr:Cargo[K_CURRENT_SELECT] := oBr:Cargo[K_LIST_O_FILES][nTopElement][K_WORK_AREA]
  SELECT (oBr:Cargo[K_CURRENT_SELECT])
  oBr:Cargo[K_CURRENT_RECNO]  := RECNO()

RETURN(NIL)

***********
* Function: NextMultiRecord()
* Author..: Troy Henikoff
* Date....: 10/18/92
* Notes...: Multi-file version of SKIP 1
* Returns.: LOGICAL - .T. if successful
***********

FUNCTION NextMultiRecord()
  LOCAL nElement
  LOCAL cMinValue := CHR(255)                    && Character value of next key
  LOCAL nNextElement                             && Element in array with next key
  LOCAL lSuccess := .F.                          && LOGICAL - .T. if a next record was found
  LOCAL nRecno                                   && temp var

  FOR nElement := 1 TO LEN(oBr:Cargo[K_LIST_O_FILES])
    SELECT (oBr:Cargo[K_LIST_O_FILES][nElement][K_WORK_AREA])
    nRecno := RECNO()

    IF SELECT() == oBr:Cargo[K_CURRENT_SELECT]   && If this is the "current record"
      SKIP 1                                     &&   Go to "next" record in file
    ENDIF

    IF .NOT. EOF() .AND. UPPER(Name) < cMinValue
      cMinValue    := UPPER(Name)
      nNextElement := nElement
      lSuccess     := .T.                        && Found one!
    ENDIF

    GOTO nRecno
  NEXT nElement

  IF lSuccess
    *** No longer current record, "skip" it to next available record ***
    SELECT (oBr:Cargo[K_CURRENT_SELECT])         && old select
    GOTO oBr:Cargo[K_CURRENT_RECNO]              && old record
    SKIP 1                                       && goto next available record

    *** Set "current" record ***
    oBr:Cargo[K_CURRENT_SELECT] := oBr:Cargo[K_LIST_O_FILES][nNextElement][K_WORK_AREA]
    SELECT (oBr:Cargo[K_CURRENT_SELECT])         && current select
    oBr:Cargo[K_CURRENT_RECNO]  := RECNO()       && goto current record
  ENDIF
RETURN(lSuccess)

***********
* Function: PrevMultiRecord()
* Author..: Troy Henikoff
* Date....: 10/18/92
* Notes...: Multi-file version of SKIP -1
* Returns.: LOGICAL - .T. if successful
***********

FUNCTION PrevMultiRecord()
  LOCAL nElement
  LOCAL cMaxValue := CHR(1)                      && Character value of next key
  LOCAL nNextElement                             && Element in array with next key
  LOCAL lSuccess := .F.                          && LOGICAL - .T. if a next record was found
  LOCAL nRecno
  LOCAL nNextRecno

  FOR nElement := 1 TO LEN(oBr:Cargo[K_LIST_O_FILES])
    SELECT (oBr:Cargo[K_LIST_O_FILES][nElement][K_WORK_AREA])
    nRecno := RECNO()                            && remember current record pointer place

    SKIP -1
    IF .NOT. BOF() .AND. UPPER(Name) > cMaxValue  && IF This is the best one yet
      cMaxValue    := UPPER(Name)                &&   Store info about it
      nNextElement := nElement
      lSuccess     := .T.                        && Found one!
      nNextRecno   := RECNO()
    ENDIF
    GOTO nRecno                                  && restore pointer to correct record
  NEXT nElement

  IF lSuccess
    *** Store newly found record ***
    oBr:Cargo[K_CURRENT_SELECT] := oBr:Cargo[K_LIST_O_FILES][nNextElement][K_WORK_AREA]
    SELECT (oBr:Cargo[K_CURRENT_SELECT])
    oBr:Cargo[K_CURRENT_RECNO]  := nNextRecno
    GOTO oBr:Cargo[K_CURRENT_RECNO]
  ENDIF

RETURN(lSuccess)

***********
*           (C) Copyright 1992 Specialized Systems & Software Inc.
*                                             All Rights Reserved
* Function: DispCustomer()
* Author..: Troy Henikoff
* Date....: 10/18/92
* Notes...: Displays a customer record for the multi-file browse
* Returns.: STRING of customer data
***********

FUNCTION DispCustomer()
  LOCAL cString                                  && string to return to be displayed
  LOCAL nRecno                                   && Current record number

  SELECT (oBr:Cargo[K_CURRENT_SELECT])           && Select correct file
  nRecno := RECNO()                              && Remember where we are
  GOTO oBr:Cargo[K_CURRENT_RECNO]                && goto where we need to be to display data
  cString := Customer + " : " + Name + " " +;    && build string to be displayed
  oBr:Cargo[K_LIST_O_FILES][ oBr:Cargo[K_CURRENT_SELECT] ][K_REP]
  GOTO nRecno                                    && go back to where we were
RETURN(cString)


***********
* Function: AdjustRecnos()
* Author..: Troy Henikoff
* Date....: 10/24/92
* Notes...: When a record gets "moved" in the index order, we must adjust the
*         : other selects' record pointers too!
***********

FUNCTION AdjustRecnos()
  LOCAL nElement
  LOCAL nSelect := SELECT()
  LOCAL lBeforeCurrent:=.T.                      && LOGICAL that indicates we are on an array element that is "before" the currently selected work area
  LOCAL cCurrentKey
  LOCAL cNextKey

  SELECT (oBr:Cargo[K_CURRENT_SELECT])
  cCurrentKey := UPPER( Name )
  cNextKey    := LEFT( cCurrentKey, LEN( cCurrentKey ) - 1 ) + CHR( ASC( RIGHT( cCurrentKey, 1) ) + 1)

  FOR nElement := 1 TO LEN(oBr:Cargo[K_LIST_O_FILES])
    SELECT (oBr:Cargo[K_LIST_O_FILES][nElement][K_WORK_AREA])
    DO CASE
      CASE SELECT() == nSelect
        *** Do nothing, we are on the correct record ***
        lBeforeCurrent := .F.                    && Now we are "after" the current select...

      CASE lBeforeCurrent
        DBSEEK( cNextKey, .T.)                   && Softseek to just beyond current key

      OTHERWISE
        DBSEEK( cCurrentKey, .T.)                && Softseek to this key or beyond

    ENDCASE
  NEXT nElement
  SELECT(nSelect)

RETURN(NIL)
