#include "inkey.ch"

#define K_CURRENT_LINE    1                     && Counter of which line you are on for multi-line-per-record TBrowse
#define K_SINGLE_LINE     2                     && Flag whether the multi-line-per-record TBrowse is in single line mode or multi-line-mode

#define K_CARGO_ELEMENTS  2

STATIC oBr
STATIC lBufferFix := .F.

FUNCTION main(xParam)
  LOCAL nRow    := ROW()
  LOCAL nCol    := COL()
  LOCAL cScreen := SAVESCREEN(0, 0, MAXROW(), MAXCOL() )

  SETCOLOR("+BG/B")
  CLEAR
  USE Customer INDEX Customer NEW SHARED
  LoadBrowse()

  IF PCOUNT() > 0
    lBufferFix := .T.
    @ 2, 20 SAY PADC("*** NO CACHING ***", 40 ) COLOR "+GR/B"
  ENDIF

  SET CURSOR OFF
  KeyProcess()
  SET CURSOR ON

  CLOSE DATABASES
  RESTSCREEN(0, 0, MAXROW(), MAXCOL(), cScreen)
  SET COLOR TO
  DEVPOS(nRow, nCol)
RETURN(NIL)

***********
* Function: LoadBrowse()
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Create & loads a TBrowse object
* Returns.: NIL
***********

FUNCTION LoadBrowse()

  SETCOLOR("+BG/N")
  oBr := TBrowseNew(4, 20, 19, 60 )

  oBr:SkipBlock             := {|x| MultiSkipper(x) }  && assign it a SkipBlock
  oBr:Cargo                 := ARRAY(K_CARGO_ELEMENTS)
  oBr:Cargo[K_CURRENT_LINE] := 1                 && Make the current line be 1, we are one the first line of the first address
  oBr:Cargo[K_SINGLE_LINE]  := .T.               && default to single-line mode
  oBr:AddColumn(TBColumnNew("Cust--   Name-----------------", {|| MultiLineDisp() } ))  &&   assign it a single column

  @ 1, 20 SAY PADC("BROWSE OF CUSTOMER.DBF",    40 ) COLOR "+GR/B"
  @22, 20 SAY PADC("F8:Toggle multi-line mode", 40 ) COLOR "+W/B"
RETURN(NIL)


***********
* Function: MultiSkipper()
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: SkipBlock for Multi-line TBrowse
*
* Params..: nToSkip - NUMBER of records to skip
*
* Returns.: NUMBER of records skipped
***********

FUNCTION MultiSkipper(nToSkip)
  LOCAL nSkipped, nLastGoodRec

  nSkipped   := 0                                && The number actually skipped
  nLastGoodRec := RECNO()                        && Keep the RECNO() of the last "good" record that was found

  DO CASE
    CASE nToSkip == 0                            && SKIP 0 is easy!
      SKIP 0

    CASE nToSkip > 0 .AND. RECNO() != LASTREC() + 1  && CASE skip foreward and not at phantom record yet / EOF()
      DO WHILE nSkipped < nToSkip                && Try to skip the requested number of times
        IF oBr:Cargo[K_CURRENT_LINE]<NumAddressLines()  && IF there are more address lines to display
          oBr:Cargo[K_CURRENT_LINE]++            &&   increment the address line instead of really skipping
          nSkipped++                             &&   increment the number of lines successfully skipped
        ELSE                                     && ELSE there are no more lines in this address
          SKIP 1                                 &&   move to next record

          IF EOF()                               && IF off the end of criteria (or file)
            EXIT                                 &&   EXIT out of the loop, no more to look at
          ENDIF                                  && ENDIF
          nSkipped++                             &&   increment the number of records successfully skipped
          nLastGoodRec := RECNO()                &&   make the current record the last "good" record
          oBr:Cargo[K_CURRENT_LINE] := 1         &&   now we are on the first line of the next record
        ENDIF                                    && ENDIF
      ENDDO

    OTHERWISE                                    && OTHERWISE skip backwards
      DO WHILE nSkipped > nToSkip                && Try to skip the requested number of times
        IF oBr:Cargo[K_CURRENT_LINE] > 1         && IF there are more address lines to display (up)
          oBr:Cargo[K_CURRENT_LINE]--            &&   decrement the address line instead of really skipping
          nSkipped--                             &&   decrement the number of lines successfully skipped
        ELSE                                     && ELSE there are no more lines in this address
          SKIP -1                                &&   move to next record

          IF BOF()                               && IF at BOF() or off criteria
            EXIT                                 &&   EXIT out of the loop, no more to look at
          ENDIF                                  && ENDIF
          nSkipped--                             &&   decrement the number of records successfully skipped
          nLastGoodRec := RECNO()                &&   make the current record the last "good" record
          oBr:Cargo[K_CURRENT_LINE]:=NumAddressLines()  &&   now we are on the last line of the new record
        ENDIF                                    && ENDIF
      ENDDO
  ENDCASE

  GOTO nLastGoodRec                              && No matter how many records were actually skipped, go
                                                 && to a "good" record.
RETURN(nSkipped)

***********
* Function: MultiLineDisp()
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Display customer data in multiple formats depending on the "line"
*         : that we are currently on.
*
* Returns.: STRING for TBrowse to display
***********

FUNCTION MultiLineDisp()
  LOCAL cLine                                    && Character string of the line to be returned
  LOCAL nLineToDisp := oBr:Cargo[K_CURRENT_LINE]  && Number of the current line

  DO CASE
    CASE  nLineToDisp == 1
      cLine = Customer->Customer+" : "+Customer->Name

    CASE nLineToDisp == 2
      cLine = SPACE(9)+Customer->Addr_1

    CASE nLineToDisp == 3 .AND. NumAddressLine()>3
      cLine = SPACE(9)+Customer->Addr_2

    OTHERWISE
      cLine = SPACE(9)+PAD(TRIM(Customer->City)+", "+Customer->State+" "+Customer->Zip,30)

  ENDCASE

RETURN(cLine)


***********
* Function: NumAddressLines()
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Calculates the number of lines in a customer record
*
* Returns.: NUMBER of lines
***********

FUNCTION NumAddressLines()
  LOCAL nLines

  IF oBr:Cargo[K_SINGLE_LINE]                    && IF we are in single line mode
    nLines := 1                                  &&   There is only ONE line no matter what
  ELSE                                           && ELSE
    nLines := IF(EMPTY(Customer->Addr_2), 3, 4)  &&   The number of lines depends on the data
  ENDIF                                          && ENDIF

RETURN(nLines)


***********
* Function: KeyProcess
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Processes keystrokes for a TBrowse
*
* Params..: none
*
* Returns.: NIL
***********

FUNCTION KeyProcess()
  LOCAL nKey                                     && Key that the user presses

  DO WHILE .T.
    nKey := 0                                    && Reset value of nKey

    *** stabilize the display (standard TBrowse stuff) ***
    DO WHILE .NOT. oBr:stabilize()               && DO while the display is not stabilized, incrementally stabilize it.
      nKey := INKEY()                            &&   check to see if the user has pressed a key
      IF nKey <> 0                               &&   IF a key was pressed
        EXIT                                     &&     EXIT out of this loop and process the user's request
      ENDIF                                      &&   ENDIF
    ENDDO                                        && ENDDO - go back and stabilize it a little more...

    *** This part is needed for multiuser applications to ensure that we are displaying the most current version of the data ***
    IF lBufferFix .AND. nKey == 0                && IF we disable caching AND the browse was completely re-drawn without a user request
      oBr:RefreshCurrent()                       &&   mark that the "Current" line needs to be refreshed
      DO WHILE .NOT. oBr:stabilize()             &&   re-draw (hopefully just 1 line)
      ENDDO                                      &&   no need to check keyboard etc.
    ENDIF                                        && ENDIF

    IF oBr:stable .AND. nKey==0                  && IF the display is stable
      nKey := INKEY(0)                           &&   Wait forever for the user to hit a key
    ENDIF                                        && ENDIF

    *** Finally, we get to process the key that was pressed ***
    DO CASE
      CASE nKey == K_DOWN
        oBr:down()

      CASE nKey == K_UP
        oBr:up()

      CASE nKey == K_PGDN
        oBr:pageDown()

      CASE nKey == K_PGUP
        oBr:pageUp()

      CASE nKey == K_ESC
        EXIT

      CASE nKey == K_ENTER .AND. oBr:Stable
        EditCustomer()
        oBr:RefreshAll()                         && re-draw entire TBrowse after editing the record

      CASE nKey==K_F8                            && IF the user pressed F8
        oBr:Cargo[K_SINGLE_LINE] := .NOT. oBr:Cargo[K_SINGLE_LINE]  && Toggle the multi-line/single-line flag
        oBr:RefreshAll()                         &&   re-draw entire TBrowse, new mode.

    ENDCASE
  ENDDO
RETURN(NIL)

***********
* Function: EditCustomer()
* Author..: Troy Henkoff
* Date....: 10/17/92
* Notes...: edits on customer record
* Returns.: NIL
***********

FUNCTION EditCustomer()
  LOCAL cScreen := SAVESCREEN(0, 0, MAXROW(), MAXCOL() )
  LOCAL cColor  := SETCOLOR( "+W/N" )
  LOCAL getlist := {}

  SET CURSOR ON
  @ 17, 1, 23, 49 BOX "Ŀ "          COLOR " BG/N"  && Draw pretty box for user
  @ 17, 4 SAY " Editing Customer Record "  COLOR "+GR/N"  && put a title on it

  IF rlock()                                     && IF we can lock the record
    @ 18,  2 SAY "Customer......:" GET Customer PICTURE "999999"  &&   Let user edit data...
    @ 19,  2 SAY "Name..........:" GET Name     PICTURE "@!"
    @ 20,  2 SAY "Address.......:" GET Addr_1
    @ 21,  2 SAY "              :" GET Addr_2
    @ 22,  2 SAY "City, St, Zip.:" GET City
    @ 22, 34                       GET State    PICTURE "!!"
    @ 22, 38                       GET Zip      PICTURE "99999-9999"
    READ
    COMMIT                                       &&   NOTE: Flush Clipper's buffers BEFORE UNLOCK
    UNLOCK                                       &&   UNLOCK the record
  ELSE                                           && ELSE
    @20,10 SAY "Record not available..." COLOR "+R/N"  &&   Someone else has the record...
    ?? CHR(7)
    INKEY(3)
  ENDIF                                          && ENDIF
  RESTSCREEN(0, 0, MAXROW(), MAXCOL(), cScreen)  && restore the screen, etc...
  SETCOLOR(cColor)
  SET CURSOR OFF

RETURN(NIL)

