#include "inkey.ch"

#define K_SHOW_BROWSE     1                      && Flag whether the browse is showing or not...
#define K_BR_TOP          2
#define K_BR_LEFT         3
#define K_BR_BOTTOM       4
#define K_BR_RIGHT        5

#define K_CARGO_ELEMENTS  5

STATIC oBr

FUNCTION main()
  LOCAL nRow    := ROW()
  LOCAL nCol    := COL()
  LOCAL cScreen := SAVESCREEN(0, 0, MAXROW(), MAXCOL() )

  SETCOLOR("+BG/B")
  CLEAR
  USE Customer INDEX Customer NEW SHARED
  LoadBrowse()

  SET CURSOR OFF
  KeyProcess()
  SET CURSOR ON

  CLOSE DATABASES
  RESTSCREEN(0, 0, MAXROW(), MAXCOL(), cScreen)
  SET COLOR TO
  DEVPOS(nRow, nCol)
RETURN(NIL)

***********
* Function: LoadBrowse()
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Create & loads a TBrowse object
* Returns.: NIL
***********

FUNCTION LoadBrowse()

  SETCOLOR("+BG/N")
  oBr := TBrowseDB(10, 23, 21, 65 )              && Generic DBF skipper included already

  oBr:Cargo                 := ARRAY(K_CARGO_ELEMENTS)
  oBr:Cargo[K_SHOW_BROWSE]  := .F.
  oBr:Cargo[K_BR_TOP]          := oBr:nTop
  oBr:Cargo[K_BR_LEFT]         := oBr:nLeft
  oBr:Cargo[K_BR_BOTTOM]       := oBr:nBottom
  oBr:Cargo[K_BR_RIGHT]        := oBr:nRight
  oBr:AddColumn(TBColumnNew("Cust--   Name-----------------", {|| Customer->Customer+" : "+Customer->Name } ))  &&   assign it a single column
  ToggleMode()                                   && draw title, boxes, etc...

RETURN(NIL)


***********
* Function: KeyProcess
* Author..: Troy Henikoff
* Date....: 10/17/92
* Notes...: Processes keystrokes for a TBrowse
*
* Params..: none
*
* Returns.: NIL
***********

FUNCTION KeyProcess()
  LOCAL nKey                                     && Key that the user presses

  DO WHILE .T.
    nKey := 0                                    && Reset value of nKey

    *** stabilize the display (standard TBrowse stuff) ***
    DO WHILE .NOT. oBr:stabilize()               && DO while the display is not stabilized, incrementally stabilize it.
      nKey := INKEY()                            &&   check to see if the user has pressed a key
      IF nKey <> 0                               &&   IF a key was pressed
        EXIT                                     &&     EXIT out of this loop and process the user's request
      ENDIF                                      &&   ENDIF
    ENDDO                                        && ENDDO - go back and stabilize it a little more...

    *** This part is needed for multiuser applications to ensure that we are displaying the most current version of the data ***
    IF nKey == 0                                 && IF we disable caching AND the browse was completely re-drawn without a user request
      oBr:RefreshCurrent()                       &&   mark that the "Current" line needs to be refreshed
      DO WHILE .NOT. oBr:stabilize()             &&   re-draw (hopefully just 1 line)
      ENDDO                                      &&   no need to check keyboard etc.
    ENDIF                                        && ENDIF

    IF oBr:stable .AND. nKey==0                  && IF the display is stable
      DispCustomer()
      nKey := INKEY(0)                           &&   Wait forever for the user to hit a key
    ENDIF                                        && ENDIF

    *** Finally, we get to process the key that was pressed ***
    DO CASE
      CASE nKey == K_DOWN
        oBr:down()

      CASE nKey == K_UP
        oBr:up()

      CASE nKey == K_PGDN
        oBr:pageDown()

      CASE nKey == K_PGUP
        oBr:pageUp()

      CASE nKey == K_ESC
        EXIT

      CASE nKey == K_ENTER .AND. oBr:Stable
        oBr:DeHiLite()
        EditCustomer()
        oBr:RefreshAll()                         && re-draw entire TBrowse after editing the record

      CASE nKey==K_F8                            && IF the user pressed F8
        ToggleMode()

    ENDCASE
  ENDDO
RETURN(NIL)

***********
* Function: EditCustomer()
* Author..: Troy Henkoff
* Date....: 10/17/92
* Notes...: edits on customer record
* Returns.: NIL
***********

FUNCTION EditCustomer()
  LOCAL cScreen := SAVESCREEN(0, 0, MAXROW(), MAXCOL() )
  LOCAL cColor  := SETCOLOR( "+W/N" )
  LOCAL getlist := {}

  SET CURSOR ON
  @ 17, 1, 23, 49 BOX "Ŀ "          COLOR " BG/N"  && Draw pretty box for user
  @ 17, 4 SAY " Editing Customer Record "  COLOR "+GR/N"  && put a title on it

  IF rlock()                                     && IF we can lock the record
    @ 18,  2 SAY "Customer......:" GET Customer PICTURE "999999"  &&   Let user edit data...
    @ 19,  2 SAY "Name..........:" GET Name     PICTURE "@!"
    @ 20,  2 SAY "Address.......:" GET Addr_1
    @ 21,  2 SAY "              :" GET Addr_2
    @ 22,  2 SAY "City, St, Zip.:" GET City
    @ 22, 34                       GET State    PICTURE "!!"
    @ 22, 38                       GET Zip      PICTURE "99999-9999"
    READ
    COMMIT                                       &&   NOTE: Flush Clipper's buffers BEFORE UNLOCK
    UNLOCK                                       &&   UNLOCK the record
  ELSE                                           && ELSE
    @20,10 SAY "Record not available..." COLOR "+R/N"  &&   Someone else has the record...
    ?? CHR(7)
    INKEY(3)
  ENDIF                                          && ENDIF
  RESTSCREEN(0, 0, MAXROW(), MAXCOL(), cScreen)  && restore the screen, etc...
  SETCOLOR(cColor)
  SET CURSOR OFF

RETURN(NIL)


***********
* Function: DispCustomer()
* Author..: Troy Henkoff
* Date....: 10/17/92
* Notes...: Displays ONE customer record
* Returns.: NIL
***********

FUNCTION DispCustomer()
  LOCAL cColor  := SETCOLOR( "BG/N" )

  DISPBEGIN()
  @ 4, 23 CLEAR TO 8, 65
  @ 4, 23      SAY "Customer...: "
  DEVOUT(Customer->Customer, "+BG/N")
  @ ROW()+1, 23 SAY "Name.......: "
  DEVOUT(Customer->Name,     "+BG/N")
  @ ROW()+1, 23 SAY "Address....: "
  DEVOUT(Customer->Addr_1,   "+BG/N")

  IF .NOT. EMPTY(Customer->Addr_2)
    DEVPOS(ROW()+1, 36)
    DEVOUT(Customer->Addr_2, "+BG/N")
  ENDIF

  DEVPOS(ROW()+1, 36)
  DEVOUT(TRIM(Customer->City)+" "+Customer->State+"  "+Customer->Zip,     "+BG/N")
  SETCOLOR(cColor)
  DISPEND()

RETURN(NIL)

***********
* Function: ToggleMode()
* Author..: Troy Henikoff
* Date....: 10/24/92
* Notes...: Toggles display mode
* Returns.: NIL
***********

FUNCTION ToggleMode()
  LOCAL cColor:=SETCOLOR("BG/B")
  DISPBEGIN()
  CLEAR
  IF oBr:Cargo[K_SHOW_BROWSE]                    && is currently showing the browse
    @ 1, 0 SAY PADC("CUSTOMER.DBF WITH RECORD DISPLAY ONLY", 80 ) COLOR "+GR/B"
    @ 3, 22, 9, 66  BOX "Ŀ "                              COLOR "BG/N"  && Draw pretty box around entire browse & display
    oBr:Cargo[K_SHOW_BROWSE]  := .F.
    oBr:nTop    := MAXROW() + 1
    oBr:nLeft   := 0
    oBr:nBottom := MAXROW() + 1
    oBr:nRight  := 1
    @ 12, 0 SAY PADC("F8:Show the Browse", 80) COLOR "+W/B"
  ELSE
    @ 1, 0 SAY PADC("BROWSE OF CUSTOMER.DBF WITH RECORD DISPLAY", 80 ) COLOR "+GR/B"
    @ 3, 22, 22, 66  BOX "Ŀ "                                   COLOR "BG/N"  && Draw pretty box around entire browse & display
    @ 9, 22 SAY "Ĵ"        COLOR "BG/N"
    oBr:nTop    := oBr:Cargo[K_BR_TOP]
    oBr:nLeft   := oBr:Cargo[K_BR_LEFT]
    oBr:nBottom := oBr:Cargo[K_BR_BOTTOM]
    oBr:nRight  := oBr:Cargo[K_BR_RIGHT]
    oBr:Cargo[K_SHOW_BROWSE]  := .T.
    @ 24, 0 SAY PADC("F8:Hide the Browse", 80) COLOR "+W/B"
  ENDIF
  oBr:Configure()
  DISPEND()
  SETCOLOR(cColor)
RETURN(NIL)
